package de.intevation.artifacts;

import org.w3c.dom.Document;

import java.io.Serializable;

/**
 * Interface of the core component of the artifact system: <strong>The artifact</strong>.
 * <br>
 *
 * An artifact is an abstract data type offering the following methods:
 *
 * <ol>
 *   <li>{@link #identifier() identifier()}: Returns a gobally unique identfier
 *        of this artifact.</li>
 *   <li>{@link #hash() hash()}: Returns a hash value over the internal state
 *        of this artifact.</li>
 *   <li>{@link #describe(Object)}: Returns a description of this artifact.</li>
 *   <li>{@link #advance(Document, Object) advance()}: Advances this artifact
 *       to the next internal state</li>
 *   <li>{@link #feed(Document, Object) feed()}: Feed new data into this artifact.</li>
 *   <li>{@link #out(Document, Object) out()}: Produces output for this artifact.</li>
 * </ol>
 *
 * @author Sascha L. Teichmann (sascha.teichmann@intevation.de)
 */
public interface Artifact
extends          Serializable
{
    /**
     * Identify this artifact.
     * @return Returns unique string to identify this artifact globally.
     */
    public String identifier();

    /**
     * Internal hash of this artifact.
     * @return Returns hash that should stay the same if the internal
     *         value has not changed. Useful for caching
     */
    public String hash();

    /**
     * A description used to build a interface to interact with this artifact.
     * @param context The global context of the runtime system.
     * @return An XML representation of the current state of the artifact.
     */
    public Document describe(Object context);

    /**
     * Change the internal state of the artifact.
     * @return An XML representation of the success of the advancing.
     * @param target Target of internal state to move to.
     * @param context The global context of the runtime system.
     */
    public Document advance(Document target, Object context);

    /**
     * Feed data into this artifact.
     * @param data Data to feed artifact with.
     * @param context The global context of the runtime system.
     * @return An XML representation of the success of the feeding.
     */
    public Document feed(Document data, Object context);

    /**
     * Produce output from this artifact.
     * @param format Specifies the format of the output.
     * @param context The global context of the runtime system.
     * @return a byte representation of the output.
     */
    public byte [] out(Document format, Object context);
}
// vim:set ts=4 sw=4 si et sta sts=4 fenc=utf8:
